/**@name LIBRARY: cietConfigCRYOGTW.ctl

@author: Herve Milcent(AB/CO)

Creation Date: 25/07/2005

*/

#uses "cietCRYOGTW.ctl"

/**
Purpose: check the coherency of the delete command

Parameters:
  dsConfig, dyn_string, input, config dyn_string
  dsDeleteDps, dyn_string, output, list of dp that will be deleted
  exceptionInfo, dyn_string, output, for errors

*/
cietConfigCRYOGTW_checkDeleteCommand(dyn_string dsConfig, dyn_string &dsDeleteDps, dyn_string &exceptionInfo) {
    string sFeName, sPLCSubApplication = "", sObjectType = "", sNumber = "", sDpPlcName, sPlcModPlc;
    dyn_string dsAlarmDps;

    if (dynlen(dsConfig) >= 2) {			// At least, delete instruction and plc name
        // get PLC Name
        sFeName = dsConfig[2];
        if ((sFeName != "") && (dsConfig[1] == UN_DELETE_COMMAND)) {
            if (dynlen(dsConfig) >= 3) {
                // get Application
                sPLCSubApplication = dsConfig[3];
            }
            if (dynlen(dsConfig) >= 4) {
                // get DeviceType
                sObjectType = dsConfig[4];
            }
            if (dynlen(dsConfig) >= 5) {
                // get device number
                sNumber = dsConfig[5];
            }
            unGenericDpFunctions_getPlcDevices("", sFeName, sPLCSubApplication, sObjectType, sNumber, dsDeleteDps);
            // if only delete key word then delete also modbus and systemalarm
            if ((sFeName != "") && (sPLCSubApplication == "") && (sObjectType == "") && (sNumber == "")) {
                // sDpPlc is CRYOGTW_plcName
                sDpPlcName = c_cietSystemIntegrity_CRYOGTW + sFeName;
                if (dpExists(sDpPlcName)) {
                    dynAppend(dsDeleteDps, sDpPlcName);
                    dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + "*" + substr(sDpPlcName, strpos(sDpPlcName, ":") + 1) + "*", c_unSystemAlarm_dpType);
                    dynAppend(dsDeleteDps, dsAlarmDps);
                }
            }
        } else {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkDeleteCommand:" + getCatStr("unGeneration", "BADINPUTS"), "");
        }
    } else {
        fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkDeleteCommand:" + getCatStr("unGeneration", "BADINPUTS"), "");
    }
}

/**
Purpose: check the coherency of the CRYOGTW front-end config

Parameters:
  currentObject: string, input, type of config
  currentLineSplit: dyn_string, input, config
  dsDeleteDps: dyn_string, input, dp to be deleted
  exceptionInfo: dyn_string, output, for errors
*/
cietConfigCRYOGTW_checkFrontEnd(string currentObject, dyn_string currentLineSplit, dyn_string dsDeleteDps, dyn_string &exceptionInfo) {
    switch (currentObject) {
        case UN_PLC_COMMAND:
            cietConfigCRYOGTW_checkApplication(currentLineSplit, dsDeleteDps, exceptionInfo);
            break;
        case UN_FESYSTEMALARM_COMMAND:
            unConfigGenericFunctions_checkFESystemAlarm(currentLineSplit, exceptionInfo);
            break;
        default:
            fwException_raise(exceptionInfo, "ERROR", getCatStr("unGeneration", "UNKNOWNFUNCTION"), "");
            break;
    }
}

/**
Purpose: delete function

Parameters:
  dsConfig, dyn_string, input, config dyn_string
  exceptionInfo, dyn_string, output, for errors
*/
cietConfigCRYOGTW_deleteCommand(dyn_string dsConfig, dyn_string &exceptionInfo) {
    string sPLCSubApplication = "", sObjectType = "", sNumber = "";
    dyn_string dsDpeList, exceptionInfoTemp;

    cietConfigCRYOGTW_checkDeleteCommand(dsConfig, dsDpeList, exceptionInfoTemp);
    if (dynlen(exceptionInfoTemp) > 0) {
        dynAppend(exceptionInfo, exceptionInfoTemp);
        fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_deleteCommand:" + getCatStr("unGeneration", "BADINPUTS"), "");
    } else {
        if (dynlen(dsConfig) >= 2) {			// At least, delete instruction and plc name
            if (dynlen(dsConfig) >= 3) {
                sPLCSubApplication = dsConfig[3];
            }
            if (dynlen(dsConfig) >= 4) {
                sObjectType = dsConfig[4];
            }
            if (dynlen(dsConfig) >= 5) {
                sNumber = dsConfig[5];
            }

            unGenericDpFunctions_deletePlcDevices(dsConfig[2], sPLCSubApplication, sObjectType, sNumber, "progressBar", exceptionInfo);
            if ((dsConfig[2] != "") && (sPLCSubApplication == "") && (sObjectType == "") && (sNumber == "")) {
                cietConfigCRYOGTW_deleteFrontEnd(dsConfig[2], exceptionInfo);
            }
        }
    }
}

/**
Purpose: set CRYOGTW front-end config

Parameters:
  currentObject: string, input, type of config
  currentLineSplit: dyn_string, input, config
  exceptionInfo: dyn_string, output, for errors
*/
cietConfigCRYOGTW_setFrontEnd(string currentObject, dyn_string currentLineSplit, dyn_string &exceptionInfo) {
    string currentDevice, sPlcName, sPlcType, sAnalogArchive;
    int iPlcNumber, driver;
    bool bEvent16;

    switch (currentObject) {
        case UN_PLC_COMMAND:
            cietConfigCRYOGTW_setApplication(currentLineSplit, exceptionInfo);
            break;

        case UN_FESYSTEMALARM_COMMAND:
            if (dynlen(currentLineSplit) == CRYO_CRYOGTW_SETFESYSTEM_ALARM_LENGTH) {
                currentDevice = currentLineSplit[CRYO_CRYOGTW_FESYSTEMALARM_DEVICENAME];
                sPlcName = currentLineSplit[CRYO_CRYOGTW_FESYSTEMALARM_PLCNAME];
                sPlcType = currentLineSplit[CRYO_CRYOGTW_FESYSTEMALARM_TYPE];
                iPlcNumber = currentLineSplit[CRYO_CRYOGTW_FESYSTEMALARM_PLCNUMBER];
                driver = currentLineSplit[CRYO_CRYOGTW_FESYSTEMALARM_DRIVER];
                bEvent16 = currentLineSplit[CRYO_CRYOGTW_FESYSTEMALARM_EVENT16];
                sAnalogArchive = currentLineSplit[CRYO_CRYOGTW_FESYSTEMALARM_ARCHIVE];

                unConfigGenericFunctions_setFESystemAlarm(currentLineSplit, makeDynString(currentDevice, sPlcName, sPlcType, iPlcNumber, driver, bEvent16, sAnalogArchive), exceptionInfo);
            } else {
                fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_setFrontEnd: " + getCatStr("unGeneration", "BADINPUTS"), "");
            }
            break;

        default:
            fwException_raise(exceptionInfo, "ERROR", getCatStr("unGeneration", "UNKNOWNFUNCTION"), "");
            break;
    }
}

/**
Purpose: check data for a sub application (not comm)

Parameters:
		dsPLC: dyn_string, input, CRYOGTW configuration line in file + additional parameters
			dsPLC must contains :
			1. string, "QUANTUM" or "PREMIUM"
			2. string, PLC name
			3. string, PLC subapplication
			4. int, PLC address = PLC number
			(additional parameters)
			5. driver number
			6. boolean archive name
			7. analog archive name
		dsDeleteDps, dyn_string, input, datapoints that will be deleted before importation
		exceptionInfo: dyn_string, output, for errors
*/
cietConfigCRYOGTW_checkApplication(dyn_string dsPLC, dyn_string dsDeleteDps, dyn_string &exceptionInfo) {
    dyn_string dsPLCLine, dsPLCAdditionalData;
    int i, iRes, iDriverNum;
    string sPLCDp, sHostname, sAliasDp, tempDp;

    if (dynlen(dsPLC) == (CRYO_CONFIG_CRYOGTW_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH)) {
        dsPLCLine = dsPLC;
        for (i = 1; i <= UN_CONFIG_PLC_ADDITIONAL_LENGTH; i++) {
            dynAppend(dsPLCAdditionalData, dsPLCLine[CRYO_CONFIG_CRYOGTW_LENGTH + 1]);
            dynRemove(dsPLCLine, CRYO_CONFIG_CRYOGTW_LENGTH + 1);
        }
        cietConfigCRYOGTW_checkData(dsPLCLine, exceptionInfo);
        cietConfigCRYOGTW_checkAdditionalData(dsPLCAdditionalData, exceptionInfo);

        // first check if the PLC name is correct:
        tempDp = dsPLCLine[UN_CONFIG_PLC_NAME];
        if (!unConfigGenericFunctions_nameCheck(tempDp)) {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkApplication: " + dsPLCLine[UN_CONFIG_PLC_NAME] + getCatStr("unGeneration", "BADPLCNAME"), "");
            return;
        }

        // check if the PLC Application Name is correct:
        tempDp = dsPLCLine[UN_CONFIG_PLC_SUBAPPLICATION];
        if (!unConfigGenericFunctions_nameCheck(tempDp)) {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkApplication: " + dsPLCLine[UN_CONFIG_PLC_SUBAPPLICATION] + getCatStr("unGeneration", "BADPLCAPPLICATIONNAME"), "");
            return;
        }

        // sPLCDp is CRYOGTW_plcName
        sPLCDp = getSystemName() +  c_cietSystemIntegrity_CRYOGTW + dsPLCLine[UN_CONFIG_PLC_NAME];
        sAliasDp = unGenericDpFunctions_dpAliasToName(dsPLCLine[UN_CONFIG_PLC_NAME]);

        if (substr(sAliasDp, strlen(sAliasDp) - 1) == ".") {
            sAliasDp = substr(sAliasDp, 0, strlen(sAliasDp) - 1);
        }
        if ((sAliasDp != "") && (sAliasDp != sPLCDp)) {
            if (dynContains(dsDeleteDps, sAliasDp) <= 0) {	// Alias exists, is not used by the "CRYOGTW" config datapoint and will not be deleted
                fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkApplication: " + dsPLCLine[UN_CONFIG_PLC_NAME] + getCatStr("unGeneration", "BADHOSTNAMEALIAS") + sAliasDp, "");
            }
        }
        if (dpExists(sPLCDp) && (dynContains(dsDeleteDps, sPLCDp) <= 0)) {
            sHostname = unGenericDpFunctions_getAlias(sPLCDp);
            iRes = dpGet(sPLCDp + ".communication.driver_num", iDriverNum);
            if (iRes >= 0) {
                if (iDriverNum != dsPLCAdditionalData[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM]) {
                    fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkApplication: " + getCatStr("unGeneration", "BADEXISTINGPLCDRIVER"), "");
                }
                if (sHostname != dsPLCLine[UN_CONFIG_PLC_NAME]) {
                    fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkApplication: " + getCatStr("unGeneration", "BADEXISTINGPLCHOSTNAME"), "");
                }
            }
        }
    } else {
        fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkApplication: " + getCatStr("unGeneration", "BADINPUTS"), "");
    }
}

/**
Purpose: check PLC data for configuration

Parameters:
		dsPLC: dyn_string, input, PLC configuration line in file
			dsPLC must contains :
			1. string, "QUANTUM" or "PREMIUM"
			2. string, name
			3. string, subApplication
			4. int, PLC address = PLC number
		exceptionInfo, dyn_string, output, for errors
*/
cietConfigCRYOGTW_checkData(dyn_string dsPLC, dyn_string &exceptionInfo) {
    unsigned uPLCNumber;

    if (dynlen(dsPLC) != CRYO_CONFIG_CRYOGTW_LENGTH) {
        fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkData:" + getCatStr("unGeneration", "BADINPUTS"), "");
    } else {
        unConfigGenericFunctions_checkPLCType(dsPLC[UN_CONFIG_PLC_TYPE], exceptionInfo);
        if (strrtrim(dsPLC[UN_CONFIG_PLC_NAME]) == "") {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkData:" + getCatStr("unGeneration", "ERRORPLCNAME"), "");
        }
        if (strrtrim(dsPLC[UN_CONFIG_PLC_SUBAPPLICATION]) == "") {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkData:" + getCatStr("unGeneration", "ERRORSUBAPPLI"), "");
        }
    }
}

/**
Purpose: check additional data for configuration

Parameters:
		dsPLC: dyn_string, input, PLC additional data
			dsPLC must contains :
			1. unsigned, driver number
			2. string, boolean archive
			3. string, analog archive
		exceptionInfo, dyn_string, output, for errors
*/
cietConfigCRYOGTW_checkAdditionalData(dyn_string dsPLC, dyn_string &exceptionInfo) {
    unsigned uPLCDriver;

    if (dynlen(dsPLC) != UN_CONFIG_PLC_ADDITIONAL_LENGTH) {
        fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkAdditionalData:" + getCatStr("unGeneration", "BADINPUTS"), "");
    } else {
        unConfigGenericFunctions_checkUnsigned(dsPLC[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM], uPLCDriver, exceptionInfo);
        if (dsPLC[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL] == "") {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkAdditionalData: bool archive" + getCatStr("unGeneration", "ERRORARCHIVEEMPTY"), "");
        }
        if (dsPLC[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_ANALOG] == "") {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkAdditionalData: analog archive" + getCatStr("unGeneration", "ERRORARCHIVEEMPTY"), "");
        }
        if (dsPLC[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_EVENT] == "") {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_checkAdditionalData: event archive" + getCatStr("unGeneration", "ERRORARCHIVEEMPTY"), "");
        }
    }
}

/**
Purpose: set PLC configuration for a sub application (not comm)

Parameters:
		dsPLC: dyn_string, input, PLC configuration line in file + additional parameters
			dsPLC must contains :
			(Plc config line)
			1. string, "CRYOGTW"
			2. string, front-end name
			3. string, front-end application
			4. string, CMW PRE-FIX
			5. Device to look for aliveCounter
			6. Specs version
			(additional parameters)
			7. driver number
			8. boolean archive name
			9. analog archive name
		exceptionInfo: dyn_string, output, for errors
*/
cietConfigCRYOGTW_setApplication(dyn_string dsPLCData, dyn_string &exceptionInfo) {
    int iPLCNumber, iRes, i, position, iDriverNumber;
    string sDpName, addressReference, tempDp, sSection, sDriverLine;
    dyn_string dsPLC, dsAdditionalParameters, dsSubApplications, dsImportTimes, dsBoolArchives, dsAnalogArchives, exceptionInfoTemp;
    dyn_string dsEntry, dsEventArchives;
    dyn_string dsCMWSection;
    string sFipNetwork;


    // Check
    if (dynlen(dsPLCData) == (CRYO_CONFIG_CRYOGTW_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH)) {
        // 1. Separate config file info and additional parameters
        dsPLC = dsPLCData;
        dynClear(dsAdditionalParameters);
        for (i = 1; i <= UN_CONFIG_PLC_ADDITIONAL_LENGTH; i++) {
            dynAppend(dsAdditionalParameters, dsPLC[CRYO_CONFIG_CRYOGTW_LENGTH + 1]);
            dynRemove(dsPLC, CRYO_CONFIG_CRYOGTW_LENGTH + 1);
        }

        // check if the PLC Application Name is correct:
        tempDp = dsPLCData[UN_CONFIG_PLC_SUBAPPLICATION];
        if (!unConfigGenericFunctions_nameCheck(tempDp)) {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_setApplication: " + dsPLCData[UN_CONFIG_PLC_SUBAPPLICATION] + getCatStr("unGeneration", "BADPLCAPPLICATIONNAME"), "");
            return;
        }

        // 3. Set PLC configuration in CRYOGTW
        sDpName = c_cietSystemIntegrity_CRYOGTW + dsPLC[UN_CONFIG_PLC_NAME];
        tempDp = sDpName;
        unConfigGenericFunctions_createDp(sDpName, CRYO_CRYOGTW_DPTYPE, exceptionInfoTemp);
        if (dynlen(exceptionInfoTemp) <= 0) {
            sDpName = getSystemName() + sDpName;
            unConfigGenericFunctions_setAlias(sDpName, dsPLC[UN_CONFIG_PLC_NAME], exceptionInfo);
            iRes = dpGet(sDpName + ".configuration.subApplications", dsSubApplications,
                         sDpName + ".configuration.importTime", dsImportTimes,
                         sDpName + ".configuration.archive_bool", dsBoolArchives,
                         sDpName + ".configuration.archive_analog", dsAnalogArchives,
                         sDpName + ".configuration.archive_event", dsEventArchives);
            position = dynContains(dsSubApplications, dsPLC[UN_CONFIG_PLC_SUBAPPLICATION]);
            if (position <= 0) {
                position = dynlen(dsSubApplications) + 1;
            }
            dsSubApplications[position] = dsPLC[UN_CONFIG_PLC_SUBAPPLICATION];
            dsImportTimes[position] = getCurrentTime();
            dsBoolArchives[position] = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL];
            dsAnalogArchives[position] = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_ANALOG];
            dsEventArchives[position] = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_EVENT];
            iRes = dpSetWait(sDpName + ".configuration.importTime", dsImportTimes,
                             sDpName + ".configuration.archive_bool", dsBoolArchives,
                             sDpName + ".configuration.archive_analog", dsAnalogArchives,
                             sDpName + ".configuration.archive_event", dsEventArchives,
                             sDpName + ".configuration.subApplications", dsSubApplications,
                             sDpName + ".communication.driver_num", dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM]);
            if (iRes < 0) {
                fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_setApplication: " + getCatStr("unPVSS", "DPSETFAILED"), "");
            }

            // CRYO.CBWMB-SH8-QL8C$Data$aliveCounter/timestamp
            // CRYO$propFE$tag
            sFipNetwork = dsPLC[CRYO_CONFIG_CRYOGTW_FIPSEGMENT];
            // sFipNetwork="CBWMB-SH8-QL8C";
            fwPeriphAddress_set(sDpName + ".communication.counter",
                                makeDynString(fwPeriphAddress_TYPE_CMW,
                                              dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM],
                                              dsPLC[CRYO_CONFIG_CRYOGTW_DEVICE] + "." + sFipNetwork + "$Data$aliveCounter/timeStamp",
                                              DPATTR_ADDR_MODE_INPUT_SPONT,
                                              1004,
                                              true,
                                              "",
                                              "",
                                              "",
                                              "",
                                              false,
                                              0,
                                              0,
                                              0,
                                              "",
                                              "66"),
                                exceptionInfo);

            iRes = dpSetWait(sDpName + ".version.PVSS", dsPLC[CRYO_CONFIG_CRYOGTW_CIET_VERSION]);
            if (iRes < 0) {
                fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_setApplication: ", "Check PVSS version float number");
            }

            // 4. Create _UnSystemAlarm datapoints
            unSystemIntegrity_createSystemAlarm(tempDp,
                                                DS_pattern,
                                                getCatStr("unSystemIntegrity", "PLC_DS_DESCRIPTION") + dsPLC[UN_CONFIG_PLC_NAME] + " -> DS driver " + dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM],
                                                exceptionInfo);
            fwArchive_set(c_unSystemAlarm_dpPattern + DS_pattern + tempDp + ".alarm",
                          dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL],
                          DPATTR_ARCH_PROC_SIMPLESM,
                          DPATTR_COMPARE_OLD_NEW,
                          0,
                          0,
                          exceptionInfo);

            unSystemIntegrity_createSystemAlarm(tempDp,
                                                DS_Time_pattern,
                                                getCatStr("unSystemIntegrity", "PLC_DS_TIME_DESCRIPTION") + dsPLC[UN_CONFIG_PLC_NAME] + " -> DS driver " + dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM],
                                                exceptionInfo);
            fwArchive_set(c_unSystemAlarm_dpPattern + DS_Time_pattern + tempDp + ".alarm",
                          dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL],
                          DPATTR_ARCH_PROC_SIMPLESM,
                          DPATTR_COMPARE_OLD_NEW,
                          0,
                          0,
                          exceptionInfo);

            // 5. CMW client config
            iDriverNumber = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM];
            iRes = fwInstallation_getSection( "CMWClient_" + iDriverNumber, dsCMWSection);
            if (dynlen(dsCMWSection) <= 0) {
                unCMWClient_setDriverConfig(iDriverNumber, exceptionInfo);
            }
        } else {
            dynAppend(exceptionInfo, exceptionInfoTemp);
        }
    } else {
        fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_setApplication: " + getCatStr("unGeneration", "BADINPUTS"), "");
    }
}

/**
Purpose: delete Front-end config

Parameters :
	sPlcName, string, input, plc name
	exceptionInfo, dyn_string, output, for errors
*/
cietConfigCRYOGTW_deleteFrontEnd(string sPlcName, dyn_string& exceptionInfo) {
    string sDpPlcName, sPlcModPlc = "", exceptionText;
    int i, length, iRes;
    dyn_string dsAlarmDps;


    sDpPlcName = c_cietSystemIntegrity_CRYOGTW + sPlcName;
    if (!dpExists(sDpPlcName)) {
        sDpPlcName = "";
    }

    if (sDpPlcName != "") {
        if (unGenericDpFunctions_getSystemName(sDpPlcName) == getSystemName()) {
            // 2. Delete _Un_Plc
            iRes = dpDelete(substr(sDpPlcName, strpos(sDpPlcName, ":") + 1));
            if (iRes < 0) {
                fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_deleteFrontEnd: " + substr(sDpPlcName, strpos(sDpPlcName, ":") + 1) + getCatStr("unGeneral", "UNOBJECTNOTDELETED"), "");
            }
            // 3. Delete alarm system
            dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + "*" + substr(sDpPlcName, strpos(sDpPlcName, ":") + 1) + "*", c_unSystemAlarm_dpType);
            length = dynlen(dsAlarmDps);
            for (i = 1; i <= length; i++) {
                _unSystemIntegrity_modifyApplicationAlertList(dsAlarmDps[i], false, exceptionInfo);
                iRes = dpDelete(dsAlarmDps[i]);
                if (iRes < 0) {
                    fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_deleteFrontEnd: " + dsAlarmDps[i] + getCatStr("unGeneral", "UNOBJECTNOTDELETED"), "");
                }
            }

        } else {
            fwException_raise(exceptionInfo, "ERROR", "cietConfigCRYOGTW_deleteFrontEnd: " + sDpPlcName + getCatStr("unGeneral", "UNOBJECTNOTDELETEDBADSYSTEM"), "");
        }
    }
}

/**
Purpose: check FE system alarm config line for CRYOGTW front-end

Parameters:
		dsConfig, dyn_string, input, config dyn_string
		exceptionInfo, dyn_string, output, for errors
*/
CRYOGTW_checkFESystemAlarm(dyn_string dsConfig, dyn_string &exceptionInfo) {
    if (dynlen(dsConfig) == CRYO_CRYOGTW_CHECKFESYSTEM_ALARM_LENGTH) {
    } else {
        fwException_raise(exceptionInfo, "ERROR", "CRYOGTW_checkFESystemAlarm:" + getCatStr("unGeneration", "BADINPUTS"), "");
    }
}

/**
Purpose: set FEsystem alarm config line form CRYOGTW front-end

Parameters:
		dsConfig, dyn_string, input, config dyn_string
		dsAdditionalParameters, dyn_string, input, additional parameters
			1. dp name
			2. plc name
			3. plc type
			4. plc number
			5. driver num
		dsAddress, dyn_string, output, for address
*/
CRYOGTW_setFESystemAlarm(dyn_string dsConfig, dyn_string dsAdditionalParameters, dyn_string &dsAddress) {
    string sDpName, sPlcName, sPLCType, addressReference, sArchive;
    int iPLCNumber, driverNum;
    string sCMWDevice;

    if (dynlen(dsConfig) == CRYO_CRYOGTW_SETFESYSTEM_ALARM_LENGTH) {
        sPLCType = dsAdditionalParameters[UN_CONFIG_ADDITIONAL_FESYSTEMALARM_TYPE];
        iPLCNumber = dsAdditionalParameters[UN_CONFIG_ADDITIONAL_FESYSTEMALARM_PLCNUMBER];
        driverNum = dsAdditionalParameters[UN_CONFIG_ADDITIONAL_FESYSTEMALARM_DRIVER];
        sDpName = dsAdditionalParameters[UN_CONFIG_ADDITIONAL_FESYSTEMALARM_DEVICENAME];
        sPlcName = dsAdditionalParameters[UN_CONFIG_ADDITIONAL_FESYSTEMALARM_PLCNAME];
        sArchive = dsAdditionalParameters[UN_CONFIG_ADDITIONAL_FESYSTEMALARM_ARCHIVE];

        sCMWDevice = dsConfig[CRYO_CRYOGTW_SETFESYSTEM_ALARM_NAME];
        dsAddress = makeDynAnytype(fwPeriphAddress_TYPE_CMW,
                                   driverNum,
                                   sCMWDevice + "$Agent_status$value",
                                   2,
                                   1004,
                                   true,
                                   "",
                                   "",
                                   "",
                                   "",
                                   true,
                                   0,
                                   0,
                                   0,
                                   "",
                                   "66");
    }
}

/**
Purpose: return the list of Dps that wil be archived during the set of the CRYOGTW front-end

Parameters:
		sCommand: string, input, the current entry in the import file
		sDp: string, input, the given dp: PLC or SystemAlarm  Dp
		sFEType: string, input, the front-end type
		dsArchiveDp: dyn_string, output, list of archivedDp
*/
CRYOGTW_getFrontEndArchiveDp(string sCommand, string sDp, string sFEType, dyn_string &dsArchiveDp) {
    switch (sCommand) {
        case UN_PLC_COMMAND:
            dsArchiveDp = makeDynString(getSystemName() + c_unSystemAlarm_dpPattern + DS_pattern + sDp + ".alarm",
                                        getSystemName() + c_unSystemAlarm_dpPattern + DS_Time_pattern + sDp + ".alarm");
            break;
        case UN_FESYSTEMALARM_COMMAND:
            dsArchiveDp = makeDynString(sDp + ".alarm");
            break;
        default:
            dsArchiveDp = makeDynString();
            break;
    }
}

/**
Purpose: return the driver config of CRYOGTW front-end

Parameters:
    iDriverNumber: int, input, the driver number
    sName: string, output, the name to display
    sDriverName: string, output, the manager name
    iManType: int, output, the manager type
    sCommandLine: int, output, the command line to start the manager
*/
CRYOGTW_getFrontEndManagerConfig(int iDriverNumber, string &sName, string &sDriverName, int &iManType, string &sCommandLine) {
    string sDriverNumber;
    sprintf(sDriverNumber, "%02d", iDriverNumber);
    sName = "CMW";
    sDriverName = "WCCOACMWClient";
    iManType = DRIVER_MAN;
    sCommandLine = "-num " + iDriverNumber + " -db CMWClient_Driver_" + sDriverNumber;
}
