#uses "fwUnitTestComponentAsserts.ctl"

#uses "fwRPC/fwRPC_core.ctl"
#uses "fwRPC/fwRPC_client.ctl"
#uses "fwRPC/fwRPC_server.ctl"
#uses "fwRPC/fwRPC_pmon.ctl"
#uses "fwRPC/fwRPC_common.ctl"


#uses "test/fwRPC/utils.ctl"


private global const string test_funcname = "mock_func";
private global const string test_stubname = "mock_stub";
private global const string test_funcname_long = "mock_func_long";
private global const string test_stubname_long = "mock_stub_long";
private global const string test_funcname_add = "mock_add";
private global const string test_stubname_add = "mock_stub_add";
private global const string test_funcname_err = "mock_func_err";
private global const string test_stubname_err = "mock_stub_err";

private global const string test_funcname_large = "mock_large";
private global const string test_stubname_large = "mock_stub_large";

private global const string test_libname  = "test/fwRPC/test_fwRPC_integration.ctl";
private global const dyn_int test_paramtype = makeDynInt(DYN_FLOAT_VAR, DYN_INT_VAR);
private global const bool test_exclusive = false;
private global const string test_result = "mock_result";
private global const string _exceptiontext = "exception";

private global int _startingWorkerCount=0;

private global string _dpWorkerRb;
private global int _worker_idx;
private global int _core_index;
private global const string _core_cmd = "fwRPC/fwRPC_core_script.ctl -1 -1 false";
private global const string _worker_cmd = "fwRPC/fwRPC_worker_script.ctl";

private global anytype _ctx;


// Setup
//------------------------------------------------------------------------------
void test_fwRPC_integration_setupSuite() {
  _BEGIN_SUITE("local integration");

  fwRPC_registerFunction(test_funcname, test_stubname, test_libname);
  fwRPC_registerFunction(
    test_funcname_long,
    test_stubname_long,
    test_libname,
    -1,
    makeDynInt(ANYTYPE_VAR),
    true
  );
  fwRPC_registerFunction(test_funcname_err, test_stubname_err, test_libname);
  fwRPC_registerFunction(
    test_funcname_add,
    test_stubname_add,
    test_libname,
    -1,
    test_paramtype
  );

  fwRPC_registerFunction(
    test_funcname_large,
    test_stubname_large,
    test_libname,
    -1
  );

  dyn_mixed coninfo = fwRPC_pmon_getConInfo();
  stopWorkers(coninfo);

  _core_index = fwRPC_pmon_appendCtrlManager(coninfo, _core_cmd);

  _worker_idx = fwRPC_pmon_appendCtrlManager(coninfo, _worker_cmd);
  fwRPC_pmon_startCtrlManager(coninfo, _core_index);
  fwRPC_pmon_startCtrlManager(coninfo, _worker_idx);

  //start local mode
  string clientName = "testclient";
  fwRPC_client_startMonitor(_ctx, clientName, false);

  dyn_dyn_mixed managers;
  fwRPC_pmon_getWorkerCtrlManagers(coninfo, managers);
  _startingWorkerCount = dynlen(managers);

  _dpWorkerRb = fwRPC_common_makeDpName(
    fwRPC_RBMBX_DPT,
    fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_WORKER + "_" +
    managers[dynlen(managers)][fwRPC_PMON_MANINFO_NUM]
  );
}

void test_fwRPC_integration_teardownSuite() {
  fwRPC_client_stopMonitor(_ctx);

  //rollback worker count if we've incremented during testing
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  dyn_dyn_mixed managers;
  fwRPC_pmon_getWorkerCtrlManagers(coninfo, managers);
  int num = dynlen(managers);

  while(num > _startingWorkerCount) {
    fwRPC_pmon_deleteLastWorker(coninfo);
    num--;
  }

  clearFuncDp(test_funcname);
  clearFuncDp(test_funcname_long);
  clearFuncDp(test_funcname_err);
  clearFuncDp(test_funcname_add);
  clearFuncDp(test_funcname_large);

  stopWorkers(coninfo);

  fwRPC_pmon_stopCtrlManager(coninfo, _worker_idx);
  fwRPC_pmon_deleteCtrlManager(coninfo, _worker_idx);

  fwRPC_pmon_stopCtrlManager(coninfo, _core_index);
  fwRPC_pmon_deleteCtrlManager(coninfo, _core_index);

  _END_SUITE();
}

void test_fwRPC_integration_setup() {
  _BEGIN_TEST();
}

void test_fwRPC_integration_teardown() {
}


// Mock / helpers
//------------------------------------------------------------------------------

anytype mock_stub(anytype hdl, anytype params) {
  delay(1,200);
  return fwRPC_makeResult(hdl, true, test_result);
}

anytype mock_stub_long(anytype hdl, anytype params) {
  delay(1);
  return fwRPC_makeResult(hdl, true, test_result);
}

int errfunc() {
  string f = "sd" + n;
  return system(cmd);
}

anytype mock_stub_err(anytype hdl, anytype params) {
  if (errfunc()) {
    return fwRPC_makeResult(hdl, false, "blabla");
  }
  return fwRPC_makeResult(hdl, true, "x");
}

anytype mock_stub_add(anytype hdl, anytype params) {
  return fwRPC_makeResult(hdl, true, params[1] + params[2]);
}


anytype mock_stub_large(anytype hdl, anytype params) {
  dyn_string payload;

  string longstr = "asdfasdfas";
  for (int i=0; i<2; ++i) longstr += longstr;

  for (int i=1; i<=100000; ++i) {
    payload[i] = longstr + i;
  }

  return fwRPC_makeResult(hdl, true, payload);
}


// Tests
//------------------------------------------------------------------------------

void test_fwRPC_integration_requestAndPoll() {
  anytype hdl;
  int status;
  anytype res;
  int newStatus;
  int err;

  fwRPC_client_call(_ctx, hdl, test_funcname, "");

  int timeout = 5000;
  do {
    status = fwRPC_client_getStatus(_ctx, hdl);

    if (status == fwRPC_REQUEST_STATUS_ERROR) {
      newStatus = fwRPC_client_getResult(_ctx, hdl, res, true, -1);
      assertTrue(false, "Request error: " + res);
    }

    delay(0, 500);
    timeout -= 500;
  } while(status != fwRPC_REQUEST_STATUS_DONE && timeout > 0);

  assertTrue(timeout > 0, "timeout reached.");

  newStatus = fwRPC_client_getResult(_ctx, hdl, res, true);

  assertEqual(fwRPC_REQUEST_STATUS_DONE, newStatus, "timeout");
  assertEqual(test_result, res);
}


void test_fwRPC_integration_requestAndWait() {
  anytype hdl;
  anytype res;

  fwRPC_client_call(_ctx, hdl, test_funcname, "");

  int ret = fwRPC_client_getResult(_ctx, hdl, res, true, 5000);
  assertEqual(fwRPC_REQUEST_STATUS_DONE, ret, "timed out!");
  assertEqual(test_result, res);
}


void test_fwRPC_integration_requestAndWaitTimeout() {
  anytype hdl;
  anytype res;

  fwRPC_client_call(_ctx, hdl, test_funcname_long, "");

  int ret = fwRPC_client_getResult(_ctx, hdl, res, true, 100);
  assertEqual(fwRPC_CLIENT_E_TIMEOUT, ret, "request did not timeout");
}


void test_fwRPC_integration_requestAndWaitError() {
  anytype hdl;
  anytype res;

  fwRPC_client_call(_ctx, hdl, test_funcname_err, "");

  int ret = fwRPC_client_getResult(_ctx, hdl, res, true, 5000);
  assertEqual(fwRPC_REQUEST_STATUS_ERROR, ret, "timed out!");
  assertEqual(ERRCLASS_VAR, getType(res));
}


void test_fwRPC_integration_core_returns_error_result_for_nonexistant_func() {
  anytype hdl; anytype res;

  fwRPC_client_call(_ctx, hdl, "idontexist", "");
  
  int ret = fwRPC_client_getResult(_ctx, hdl, res, true);
  assertEqual(fwRPC_REQUEST_STATUS_ERROR, ret, "timeout");
  assertEqual(ERRCLASS_VAR, getType(res));
  assertEqual(getErrorType(fwRPC_EX_INVALID_PARAM), getErrorType(res));
}


void test_fwRPC_integration_subscribeAndRequest() {
  anytype hdl;

  fwRPC_client_call(_ctx, hdl, test_funcname, "");

  int status;
  anytype result;
  _waitForCompletion(_ctx, hdl, status, result);
  
  assertEqual(fwRPC_REQUEST_STATUS_DONE, status);
  assertEqual(test_result, result);
}


void test_fwRPC_integration_subscribeAndError() {
  anytype hdl;

  fwRPC_client_call(_ctx, hdl, test_funcname_err, "");

  int status;
  anytype result;
  _waitForCompletion(_ctx, hdl, status, result);

  assertEqual(fwRPC_REQUEST_STATUS_ERROR, status);
  assertEqual(
    fwException_getClass(fwRPC_EX_WORKER_EXECUTION),
    fwException_getClass(result)
  );
}


void test_fwRPC_integration_requestSpecificType() {
  anytype hdl;

  fwRPC_client_call(_ctx, hdl, test_funcname_add, makeDynInt(1,4));

  int status;
  anytype result;
  _waitForCompletion(_ctx, hdl, status, result);

  assertEqual(fwRPC_REQUEST_STATUS_DONE, status);
  assertEqual(5, result);
}


void test_fwRPC_integration_requestSpecificTypeError() {
  anytype hdl;

  fwRPC_client_call(_ctx, hdl, test_funcname_add, "string!!");

  int status;
  anytype result;
  _waitForCompletion(_ctx, hdl, status, result);

  assertEqual(fwRPC_REQUEST_STATUS_ERROR, status);
  assertEqual(
    fwException_getClass(fwRPC_EX_INVALID_PARAM),
    fwException_getClass(result)
  );
}


void test_fwRPC_integration_cancelRequest() {
  anytype hdl;

  fwRPC_client_call(_ctx, hdl, test_funcname_long, "");

  fwRPC_client_cancel(_ctx, hdl);
  
  int status;

  anytype result;
  _waitForCompletion(_ctx, hdl, status, result);

  assertEqual(fwRPC_REQUEST_STATUS_CANCEL, status);
}


void test_fwRPC_integration_exclusiveExecution() {
  _startJobWatch(1, _dpWorkerRb);
  anytype hdl;

  fwRPC_client_call(_ctx, hdl, test_funcname_long, "");
  fwRPC_client_call(_ctx, hdl, test_funcname_long, "");
  fwRPC_client_call(_ctx, hdl, test_funcname,      "");
  fwRPC_client_call(_ctx, hdl, test_funcname_long, "");

  int status;
  string result;
  _waitForCompletion(_ctx, hdl, status, result);

  _stopJobWatch(_dpWorkerRb);
}


void test_fwRPC_integration_listServices() {
  dyn_string services;

  int r = fwRPC_client_listServices(_ctx, services, true, 5000);

  assertEqual(0, r, "timeout");
  assert(dynlen(services) > 0, "No services were returned");
  bool found = false;
  for (int i=1; i<=dynlen(services); i++) {
    if (services[i] == test_funcname) {
      found = true;
      break;
    }
  }
  assertTrue(found, "Service was not found");
}


void test_fwRPC_integration_recovery() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();
  fwRPC_pmon_stopCtrlManager(coninfo, _worker_idx);

  anytype hdl;

  fwRPC_client_call(_ctx, hdl, test_funcname, "");

  _subscribeHdl(_ctx, hdl);
  bool ok = _waitForStatus(hdl, fwRPC_REQUEST_STATUS_QUEUED);
  assertTrue(ok, "status should reach queued before timeout");

  fwRPC_client_recover(_ctx, "mock_statusCB");

  _clearStatuses(hdl);

  ok = _waitForStatus(hdl, fwRPC_REQUEST_STATUS_QUEUED);
  assertTrue(ok, "status should reach queued before timeout (2)");

  fwRPC_pmon_startCtrlManager(coninfo, _worker_idx);
}


void test_fwRPC_integration_requests_are_reset_on_worker_death() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  anytype hdl;

  fwRPC_client_call(_ctx, hdl, test_funcname, "");
  fwRPC_client_subscribeCallStatus(_ctx, hdl, "mock_statusCB");

  bool ok = _waitForStatus(hdl, fwRPC_REQUEST_STATUS_PROCESSING);
  assertTrue(ok, "status should reach processing before timeout");

  //stop worker and trigger a request reset
  fwRPC_pmon_stopCtrlManager(coninfo, _worker_idx);

  //wait a sufficiant amount of time for workermngr to register death
  delay(fwRPC_WORKER_HEARTBEAT_WAIT_S + ((fwRPC_WORKER_HEARTBEAT_RATE_MS/1000)%60)*2);

  fwRPC_pmon_startCtrlManager(coninfo, _worker_idx);

  assertEqual(true, _getLastStatus(hdl) > 0, "status should not be none");

  ok = _waitForStatus(hdl, fwRPC_REQUEST_STATUS_DONE);
  assertTrue(ok, "status should reach done before timeout");

  dyn_int states = _getAllStatuses(hdl);

  int i = 1;
  while(states[i] != fwRPC_REQUEST_STATUS_ASSIGNED) ++i; //first might be queued
  assertEqual(fwRPC_REQUEST_STATUS_ASSIGNED,   states[i], "1 status should be ASSIGN");
  assertEqual(fwRPC_REQUEST_STATUS_PROCESSING, states[++i], "2 status should be PROCES");
  assertEqual(fwRPC_REQUEST_STATUS_QUEUED,     states[++i], "3 status should be QUEUED");
  assertEqual(fwRPC_REQUEST_STATUS_ASSIGNED,   states[++i], "4 status should be ASSIGN");
  assertEqual(fwRPC_REQUEST_STATUS_PROCESSING, states[++i], "5 status should be PROCES");
  assertEqual(fwRPC_REQUEST_STATUS_DONE,       states[++i], "6 status should be DONE");

  delay(fwRPC_WORKER_HEARTBEAT_WAIT_S + (fwRPC_WORKER_HEARTBEAT_RATE_MS/1000)%60);
}


void test_fwRPC_integration_worker_ringbuffers_are_removed_on_worker_death() {
  dyn_mixed coninfo = fwRPC_pmon_getConInfo();

  delay(1);

  assertTrue(dpExists(_dpWorkerRb), "Worker RB should exist: " + _dpWorkerRb);

  fwRPC_pmon_stopCtrlManager(coninfo, _worker_idx);

  //wait a sufficiant amount of time for workermngr to register death
  delay(fwRPC_WORKER_HEARTBEAT_WAIT_S + ((fwRPC_WORKER_HEARTBEAT_RATE_MS/1000)%60)*2);

  assertFalse(dpExists(_dpWorkerRb), "Worker RB should have been deleted: " + _dpWorkerRb);

  fwRPC_pmon_startCtrlManager(coninfo, _worker_idx);

  delay(1);
}


void test_fwRPC_integration_sends_error_state_on_bad_func_name() {
  anytype result;
  int status = fwRPC_client_callSync(_ctx, "unregistered func", "", result);

  assertEqual(fwRPC_REQUEST_STATUS_ERROR, status);
  assertEqual(
    fwException_getClass(fwRPC_EX_INVALID_PARAM),
    fwException_getClass(result)
  );
}


void test_fwRPC_integration_core_restarts_workers_when_able() {
  // TODO: this test should be revised..

  // Here we want to test the core's ability to restart the workers at a sane
  // time, when all requests are either in a queued or completed state.

  //change the accepted type to int
  fwRPC_registerFunction(
    test_funcname_long,
    test_stubname_long,
    test_libname,
    -1,
    makeDynInt(INT_VAR),
    true
  );

  anytype hdl1, hdl2, hdl3, hdl4;
  mixed res;
  int status;

  // As the worker has already been started, the new function definition will
  // not be loaded. Therefore, these two calls will succeed.
  fwRPC_client_call(_ctx, hdl1, test_funcname_long, "");
  fwRPC_client_call(_ctx, hdl2, test_funcname_long, "");

  delay(0, 100);

  // Request the restart. The two above should be in the middle of processing
  // and succeed some time in the future, before the restart is executed.
  dpSetWait(fwRPC_DBG_DP + ".restartCore", true);

  int timeout = 5000;
  status = fwRPC_client_getStatus(_ctx, hdl1);
  while (
    timeout > 0 &&
    status != fwRPC_REQUEST_STATUS_DONE &&
    status != fwRPC_REQUEST_STATUS_PROCESSING
  ) {
    status = fwRPC_client_getStatus(_ctx, hdl1);
    timeout -= 100;
    delay(0,100);
  }

  assert(timeout > 0,
    "hdl1 status should be done or processing: " + hdl1 + " (it is " + status + ")"
  );

  status = fwRPC_client_getStatus(_ctx, hdl2);
  assert((fwRPC_REQUEST_STATUS_QUEUED == status || fwRPC_REQUEST_STATUS_ASSIGNED == status),
    "hdl2 status should be queued: " + hdl2 + " it is " + status
  );


  // At this point, the worker should be idle, and the restart should begin. All
  // future requests will be enqueued at the core, while the restart is in
  // progress. Therefore, the two following requests should have a queued
  // status. We call these with the new, correct parameter types.

  fwRPC_client_call(_ctx, hdl3, test_funcname_long, ""); //this should fail
  delay(0,800);
  status = fwRPC_client_getStatus(_ctx, hdl3);
  assertEqual(fwRPC_REQUEST_STATUS_QUEUED, status,
    "hdl3 status should be queued: " + hdl3 + " (it is " + status + ")"
  );

  fwRPC_client_call(_ctx, hdl4, test_funcname_long, 1); //this should succeed
  delay(0,800);
  status = fwRPC_client_getStatus(_ctx, hdl4);
  assertEqual(fwRPC_REQUEST_STATUS_QUEUED, status,
    "hdl4 status should be queued: " + hdl4 + " (it is " + status + ")"
  );

  delay(12); //wait for the worker to restart

  int status;
  mixed res;
  int ret = _waitForCompletion(_ctx, hdl3, status, res);
  assertEqual(fwRPC_REQUEST_STATUS_ERROR, status,
    "hdl3 status should be err: " + hdl3 + " (it is " + ret + ")"
  );

  ret = fwRPC_client_getResult(_ctx, hdl4, res, true, 10000);
  assertEqual(fwRPC_REQUEST_STATUS_DONE, ret,
    "hdl4 status should be done: " + hdl4 + " (it is " + ret + ")"
  );

  ret = fwRPC_client_getResult(_ctx, hdl2, res, true, 10000);
  assertEqual(fwRPC_REQUEST_STATUS_DONE, ret,
    "hdl2 should have succeeded: " + hdl2
  );
}


void test_fwRPC_integration_handles_large_payloads() {
  dyn_string res;
  fwRPC_client_callSync(_ctx, test_funcname_large, "", res);

  assertEqual(100000, dynlen(res));
}


void test_fwRPC_integration_still_works_with_multiple_restart_tries() {
  dyn_string res;

  dpSetWait(fwRPC_DBG_DP + ".restartCore", true);
  delay(10); // If we don't wait, the sync call will expire before the restart
             // finishes
  fwRPC_client_callSync(_ctx, test_funcname, "", res);
  assertEqual(test_result, res, "first result should come through");

  delay(12);
  dpSetWait(fwRPC_DBG_DP + ".restartCore", true);
  delay(1);

  fwRPC_client_callSync(_ctx, test_funcname, "", res);
  assertEqual(test_result, res, "second result should come through");
}
