/**@name LIBRARY: unCMWServer.ctl

@author: Celine VUILLERMOZ (AB-CO-IS)

Creation Date: 07/10/2005 

Modification History: 
  28/05/2010: Herve
  - fix bug with CMWServer: integrity period in msec not in sec in the DP
  
  13/07/2009: Herve
    - set integrity period to 30sec.
    
  10/06/2009: Herve
    - RBAC setting: cmw_Policy and cmw_Class
    
  22/01/2008: Herve
    - move function unCMWServer_userCB to unCMWServerManager.pnl 
    
	07/02/2006: Herve
		- in unCMWServer_Integrity check if <= instead of ==
		
version 1.1

External Functions: 
	
Internal Functions: 
	
Purpose: 
This library contains generic functions for CMWServer component.

Usage: Public

PVSS manager usage: CTRL (WCCOACTRL), UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.

	Functions:
		unCMWServer_DefaultSettingsManager
		unCMWServer_LoadSettingsManager
		unCMWServer_getStatus
		unCMWServer_Configuration
		unCMWServer_Integrity
		unCMWServer_Check_Integrity
		unCMWServer_Update_PVSS_Status
		unCMWServer_SaveState
		unCMWServer_GetPosition
		unCMWServer_getFromFile
		unCMWServer_CheckLine
		unCMWServer_setStatusTable
		_unSystemIntegrity_CMWServer_CB	
*/

//@{

//CMWServerManager
const int CMWSERVER_MAN_CONFIGPATH=1;
const int CMWSERVER_MAN_NUMPERIOD = 2;
const int CMWSERVER_CMW_NAMES = 3;
const int CMWSERVER_CMW_SERVER = 4;
const int CMWSERVER_LOG_INTIMEPERIOD = 5;
const int CMWSERVER_LOG_SHOWSTATISTICS = 6;
const int CMWSERVER_MAN_MAXASYNCUPDATES = 7;
const int CMWSERVER_MAN_LOCKDATAPERIOD = 8;
const int CMWSERVER_CMW_DATATAGNAME = 9;
const int CMWSERVER_CMW_TIMETAGNAME =10;
const int CMWSERVER_CMW_POLICY = 11;
const int CMWSERVER_CMW_CLASS = 12;

//CMWServer State
/*
const int State_Command=1;
const int State_StateOut=2;
const int State_Message=3;
const int State_PVSS=4;
const int State_Timeout=5;
const int State_LastRefEnd=6;
*/

//Catalogue name
const string g_sCat_CMWServer="unCMWServer";

//user allowed to modify the LHCLogging if Unicos Package not installed
const string c_CMWServer_userAdmin="root"; 

// unCMWServer_DefaultSettingsManager

/**
Purpose: set the default settings for Manager DP

Parameter: 	sDP, string, input, DataPoint Name
			exceptionInfo, dyn string, output, Message

Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/

unCMWServer_DefaultSettingsManager(string sDP, dyn_string &exceptionInfo)
{
int iRes, iTimeout, iDelay;
dyn_string dsSettings;
string sFunction, sName, sSystem;

sSystem = unGenericDpFunctions_getSystemName(sDP);
unCMWServer_Configuration(dsSettings, sFunction, sName, iTimeout, exceptionInfo);
if (dynlen(exceptionInfo)==0)
	{
	if (dpExists(sDP) && dynlen(dsSettings)>=CMWSERVER_MAN_LOCKDATAPERIOD)
			{
			iRes=dpSet(sDP + ".Settings.man_ConfigPath", dsSettings[CMWSERVER_MAN_CONFIGPATH],
						sDP + ".Settings.log_ShowStatistics", dsSettings[CMWSERVER_LOG_SHOWSTATISTICS],
						sDP + ".Settings.cmw_Names", dsSettings[CMWSERVER_CMW_NAMES],
						sDP + ".Settings.cmw_Server", dsSettings[CMWSERVER_CMW_SERVER],
						sDP + ".Settings.log_InTimePeriod", dsSettings[CMWSERVER_LOG_INTIMEPERIOD],
						sDP + ".Settings.man_MaxAsyncUpdates", dsSettings[CMWSERVER_MAN_MAXASYNCUPDATES],			
						sDP + ".Settings.man_LockDataPeriod", dsSettings[CMWSERVER_MAN_LOCKDATAPERIOD],
						sDP + ".Settings.man_NumPeriod",dsSettings[CMWSERVER_MAN_NUMPERIOD],
						sDP + ".Settings.cmw_DataTagName", dsSettings[CMWSERVER_CMW_DATATAGNAME],
						sDP + ".Settings.cmw_TimeTagName", dsSettings[CMWSERVER_CMW_TIMETAGNAME],
                                                sDP + ".Settings.cmw_Policy", dsSettings[CMWSERVER_CMW_POLICY],
                                                sDP + ".Settings.cmw_Class", dsSettings[CMWSERVER_CMW_CLASS],
						sDP + ".State.st_CommandIn", 100,				//Stop
						sDP + ".State.st_StateOut", 1000,			//Unknown state
						sDP + ".State.ev_MessageOut", c_Ok);		//OK
			}
	else
		fwException_raise(exceptionInfo, "ERROR", "unCMWServer_DefaultSettingsManager(): " + getCatStr(g_sCat_CMWServer,"DPDOESNOTEXIST"),"");
	}
}

// unCMWServer_LoadSettingsManager
/**
Purpose: Get  the settings for Manager

Parameter: 	sDP, string, input, DataPoint Name
			dsParam, dyn string, output, String Parameters
			ddiParam, dyn dyn int, output, Instance Parameter
			exceptionInfo, dyn string, output, Message
			
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unCMWServer_LoadSettingsManager(string sDP, dyn_string &dsParam, dyn_string &exceptionInfoTemp)
{
int iRes, i;

if (dpExists(sDP + ".Settings"))
	{
	iRes=dpGet(sDP + ".Settings.man_ConfigPath", dsParam[CMWSERVER_MAN_CONFIGPATH],
						sDP + ".Settings.log_ShowStatistics", dsParam[CMWSERVER_LOG_SHOWSTATISTICS],
						sDP + ".Settings.cmw_Names", dsParam[CMWSERVER_CMW_NAMES],
						sDP + ".Settings.cmw_Server", dsParam[CMWSERVER_CMW_SERVER],
						sDP + ".Settings.log_InTimePeriod", dsParam[CMWSERVER_LOG_INTIMEPERIOD],
						sDP + ".Settings.man_MaxAsyncUpdates", dsParam[CMWSERVER_MAN_MAXASYNCUPDATES],			
						sDP + ".Settings.man_LockDataPeriod", dsParam[CMWSERVER_MAN_LOCKDATAPERIOD],
						sDP + ".Settings.man_NumPeriod",dsParam[CMWSERVER_MAN_NUMPERIOD],
						sDP + ".Settings.cmw_DataTagName", dsParam[CMWSERVER_CMW_DATATAGNAME],
						sDP + ".Settings.cmw_TimeTagName", dsParam[CMWSERVER_CMW_TIMETAGNAME],
                                                sDP + ".Settings.cmw_Policy", dsParam[CMWSERVER_CMW_POLICY],
                                                sDP + ".Settings.cmw_Class", dsParam[CMWSERVER_CMW_CLASS]);

	if (iRes<0)
		fwException_raise(exceptionInfoTemp, "ERROR", "unCMWServer_LoadSettingsManager(): " + getCatStr(g_sCat_CMWServer,"DPGETERROR"),"");
			
	}
else
	fwException_raise(exceptionInfoTemp, "ERROR", "unCMWServer_LoadSettingsManager(): " + getCatStr(g_sCat_CMWServer,"DPDOESNOTEXIST"),"");
	
}


// unCMWServer_getStatus
/**
Purpose: get state of CMWServer system

Parameter: 	iIndex, int, input, Position in Array  
			iState, int, output, State of Manager or Archive  
						
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0
	. operating system: Linux, NT, XP and W2000, but tested only under W2000 and XP.
	. distributed system: yes.
*/
unCMWServer_getStatus(int iIndex, int &iState)
{
int iState_1=0, iState_2=0;

iState=0;

if (dynlen(g_diCMWServer_PVSS)>=iIndex && dynlen(g_diCMWServer_Message)>=iIndex && iIndex>0) 
	{
	//PVSS
	if ((g_diCMWServer_PVSS[iIndex]!=1)) 														//Archive or Manager stopped
		iState_1=1;

	//Message
//	if ((g_diCMWServer_Message[iIndex]>10001) && (g_diCMWServer_Message[iIndex]!=10150))		//10150 -> Manager Internal Purpose
	if (g_diCMWServer_Message[iIndex]>10001)
		iState_2=g_diCMWServer_Message[iIndex];																			//Warning																									
	}

if (dynContains(g_diCMWServerCritical_Alarm,iState_2)>0 || (iState_1==1))	//Error(s)
	iState=c_Error;
else
	{
	if (iState_2>0)	
		iState=c_Warning;
	}
}


// unCMWServer_setStatusTable
/**
Purpose: set state of CMWServer system

Parameter: 	bType, bool, input, Manager or Archive
			sName, string, input, Name of Manager or Archive 
			iIndex, int, input, index for Global Variable 
			iState, int, input, Status
			tState_6, int, input, Time of Status change
			dbTimeout, dyn bool, input, True = Warning Timeout or False = Ok
						
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCMWServer_setStatusTable(string sName, int iIndex, int iState, time tState_6)
{
int i, length;
string sContent, sTable, sText_PVSS, sText_Command, sText_StateOut, sText_Message, sColor, t;
dyn_string dsSplitTemp;

	length=ManagersList.lineCount();
	sTable="ManagersList";
	
for (i=0;i<length;i++)
	{
	getValue(sTable, "cellValueRC", i, "Names", sContent);
	if (sContent==sName)			//find Line
		{
		unCoreLHCServices_PVSS(g_diCMWServer_PVSS[iIndex], sText_PVSS);
		sText_Command=getCatStr(g_sCat_CMWServer, g_diCMWServer_Command[iIndex]);
		sText_StateOut = getCatStr(g_sCat_CMWServer, g_diCMWServer_State[iIndex]);
		
//		if (g_diCMWServer_Message[iIndex]!=10150)		//->internal purpose for WCCOALaser
			sText_Message = getCatStr(g_sCat_CMWServer, g_diCMWServer_Message[iIndex]);
	/*	else
			getValue(sTable, "cellValueRC", i, "MessageOut", sText_Message);*/
				
		switch(iState)
			{
			case 0:
				sColor="unCMWServer_Ok";
				break;
			
			case c_Warning:
				sColor="unCMWServer_Warning";
				break; 
			
			case c_Error:
				sColor="unCMWServer_Bad";
				break;
			
			default:
				sColor="unCMWServer_Bad";
			}
		
		if (tState_6!="")
			{	
			t=formatTime("%c", tState_6);
	 	
	 		if (_UNIX)
				t+=timeFromGMT(); // ???
 			
 			setMultiValue (sTable, "cellBackColRC", i, "X", sColor, sTable, "cellValueRC", i, "PVSS", sText_PVSS, sTable, "cellValueRC", i, "Command", sText_Command, sTable, "cellValueRC", i, "StateOut", sText_StateOut, sTable, "cellValueRC", i, "MessageOut", sText_Message, sTable, "cellValueRC", i, "Time", t);
 			}
 		else	
			setMultiValue (sTable, "cellBackColRC", i, "X", sColor, sTable, "cellValueRC", i, "PVSS", sText_PVSS, sTable, "cellValueRC", i, "Command", sText_Command, sTable, "cellValueRC", i, "StateOut", sText_StateOut, sTable, "cellValueRC", i, "MessageOut", sText_Message);
		
		return;
		}
	}
}

// unCMWServer_Configuration
/**
Purpose: Initialize CMWServer configuration

Parameter: 	sFunction_Name, string, output, Function Name to be used to update Table for configFile
			sComponent_Name, string, output, Name of the Component used for this Manager
			iIntegrity_Timeout, int, output, Timeout to check if manager alive
			dsDefault_Settings, dyn_string, output, Return all the settings
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCMWServer_Configuration(dyn_string &dsDefault_Settings, string  &sFunction_Name, string &sComponent_Name, int &iIntegrity_Timeout, dyn_string &exceptionInfo_Temp)
{
	int iRes;
	string sSystemName =getSystemName();
	
	if (dpExists(sSystemName + "CMWServer_Info.defaultSettings"))
		{
		iRes=dpGet(sSystemName + "CMWServer_Info.defaultSettings.Settings",dsDefault_Settings);
                while(dynlen(dsDefault_Settings) < CMWSERVER_CMW_CLASS)
                  dynAppend(dsDefault_Settings, "");
		if (iRes<0)
			fwException_raise(exceptionInfo_Temp, "ERROR", "CMWServer_Configuration(): " + getCatStr(g_sCat_CMWServer,"DPSETGETFAILED"),"");	
		
		}
	else
		{
		iIntegrity_Timeout=300;
		dsDefault_Settings[CMWSERVER_MAN_CONFIGPATH]="data/CMWServerData";
		dsDefault_Settings[CMWSERVER_MAN_NUMPERIOD]="30000"; 
		dsDefault_Settings[CMWSERVER_LOG_SHOWSTATISTICS]="-m 0x00000003"; 
		dsDefault_Settings[CMWSERVER_CMW_NAMES]="pslxmcr:5020"; 
		dsDefault_Settings[CMWSERVER_CMW_SERVER]="CMWServerName"; 
		dsDefault_Settings[CMWSERVER_LOG_INTIMEPERIOD]="10"; 
		dsDefault_Settings[CMWSERVER_MAN_MAXASYNCUPDATES]="5"; 
		dsDefault_Settings[CMWSERVER_MAN_LOCKDATAPERIOD]="100"; 
		dsDefault_Settings[CMWSERVER_CMW_DATATAGNAME]="value"; 
		dsDefault_Settings[CMWSERVER_CMW_TIMETAGNAME]="time"; 
                dsDefault_Settings[CMWSERVER_CMW_POLICY] = "";
                dsDefault_Settings[CMWSERVER_CMW_CLASS] = "";
		
		iRes=dpCreate("CMWServer_Info","_Un_CMWServer_Config");
		if (iRes<0)
			{
			fwException_raise(exceptionInfo_Temp, "ERROR", "unCMWServer_Configuration(): " + getCatStr(g_sCat_CMWServer,"DPCREATEFAILURE"),"");	
			}
		else
			{																																																																					
			iRes= dpSetWait(sSystemName + "CMWServer_Info.defaultSettings.Settings");
			}		
		}
}

// unCMWServer_getPosition
/**
Purpose: get index for Manager (need for Global dyn)

Parameter: 	sName, string, input, Archive's name or Manager's name
			iIndex, int, output, position in Array of this name
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCMWServer_getPosition(string sName, int &iIndex)
{
int i, len;

	iIndex=0;
	len=dynlen(g_dsCMWServer_Name);
	for (i=1;i<=len;i++)
		{
		if (g_dsCMWServer_Name[i]==sName)
			{
			iIndex=i;
			i=len+1;
			}
		}		
	
}

// unCMWServer_Check
/**
Purpose: Check Line syntax for Config File

Parameter:	sDpe, string, input, string for the Dpe cell
			sName, string, input, string for the Name
			sState, string, input, string for the State
			sRange, string, input, string for the Range
			sFF, string, input, string for the Fault Family 
			sFM, string, input, string for the Fault Member 
			sFC, string, input, string for the Fault Code 
			sDescription, string, input, string for the Description
			sMess_Error, string, output, Pattern for the error 
			bError, bool, input, true=Error
			stable, string, input, Table name to update
			bPanelCheck, bool, input, true=Need to add line before setColor
			iRow, int, input, Line Number in Table
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCMWServer_Check(string stable, bool bPanelCheck, int iRow, string &sMess_Error, bool &bError, string Select, string Device, string Alias,string Dpe, string Property, string Direction, string Value, string Time)
{
string sError;
int iLine;

	if (Select!="S")
	{
		unCMWServer_CheckLine(Device, Alias, Dpe, Property, Direction, Value, Time, sError);

		if (sError!="")
		{
		// In case of just checking (unLaserConfigFile_check.pnl) -> need to add line in Table
			if (bPanelCheck)
			{
				setValue(stable,"appendLine",
						"Select", Select,
						"Device", Device, 
						"Alias", Alias,
						"Dpe", Dpe, 
						"Property", Property, 
						"Direction", Direction, 
						"Value", Value, 
						"Time", Time);
				getValue(stable,"lineCount",iLine);
				iLine=iLine-1;
			}											
			else iLine=iRow;
			bError=true;
		
		switch(sError)
			{
			case "Device":						
				setMultiValue(stable, "cellBackColRC", iLine, "Device", "unCMWServer_Bad");				
				sMess_Error= sMess_Error + "\n\t-> " + Alias + Dpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_CMWServer,"LOGCHECKDEVICE") + "\n";
				if (!bPanelCheck)
				{
					getValue(stable, "cellValueRC", iRow + 1, "Device", Device);
					if (Device=="")
						{
						setMultiValue(	stable, "cellBackColRC", iLine, "Device", "unCMWServer_Bad");	
						}
				}
				
			case "Alias":						
				setMultiValue(	stable, "cellBackColRC", iLine, "Alias", "unCMWServer_Bad");				
				sMess_Error= sMess_Error + "\n\t-> " + Alias + Dpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_CMWServer,"LOGCHECKALIAS") + "\n";
			break;
			
			case "Dpe":						
				setMultiValue(	stable, "cellBackColRC", iLine, "Dpe", "unCMWServer_Bad");				
				sMess_Error= sMess_Error + "\n\t-> " + Alias + Dpe  + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_CMWServer,"LOGCHECKDPE") + "\n";
				if (!bPanelCheck)
				{
					getValue(stable, "cellValueRC", iRow + 1, "Dpe", Dpe);
					if (Dpe=="")
						{
						setMultiValue(	stable, "cellBackColRC", iLine, "Dpe", "unCMWServer_Bad");	
						}
				}
			break;

			case "Property":			
				setMultiValue (stable, "cellBackColRC", iLine,"Name", "unCMWServer_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + Alias + Dpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_CMWServer,"LOGCHECKPROPERTY") + Name + "\n";
			break;
			
			case "Direction":
				setMultiValue (stable, "cellBackColRC", iLine, "Direction", "unCMWServer_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + Alias + Dpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_CMWServer,"LOGCHECKDIRECTION") + Direction + "\n";
			break;
			
			case "Value":
				setMultiValue (stable, "cellBackColRC", iLine, "Value", "unCMWServer_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + Alias + Dpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_CMWServer,"LOGCHECKVALUE") + Value + "\n";
			break;
			
			case "Time":
				setMultiValue (stable, "cellBackColRC", iLine, "Time", "unCMWServer_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + Alias + Dpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_CMWServer,"LOGCHECKTIME") + Time + "\n";
			break;
			
			default:
				setMultiValue(	stable, "cellBackColRC", iLine, "Select", "unCMWServer_Bad", 
								stable, "cellBackColRC", iLine, "Device", "unCMWServer_Bad",
								stable, "cellBackColRC", iLine, "Alias", "unCMWServer_Bad", 
								stable, "cellBackColRC", iLine, "Dpe", "unCMWServer_Bad", 
								stable, "cellBackColRC", iLine,"Property", "unCMWServer_Bad", 
								stable, "cellBackColRC", iLine, "Direction", "unCMWServer_Bad", 
								stable, "cellBackColRC", iLine, "Value", "unCMWServer_Bad", 
								stable, "cellBackColRC", iLine, "Time", "unCMWServer_Bad",
								stable, "cellValueRC", iLine, "Select", "F");
			}
		}
		else
		{
			if ((Select=="Q")&&(Dpe!="")&&(Alias==""))
			{
				if (bPanelCheck)
				{
					setValue(stable,"appendLine",
							"Select", Select,
							"Device", Device, 
							"Alias", Alias, 
							"Dpe", Dpe, 
							"Property", Property, 
							"Direction", Direction, 
							"Value", Value, 
							"Time", Time);
				
					getValue(stable,"lineCount",iLine);
					iLine=iLine-1;
					setMultiValue(stable, "cellBackColRC", iLine, "Dpe", "unCMWServer_NotDefined");
				}											
				bError=true;
				sMess_Error= sMess_Error + "\n\t-> " + Alias + Dpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_CMWServer,"LOGCHECKNOTDEFINED") + "\n";
			}
		}					
	}


}

// unCMWServer_CheckLine
/**
Purpose: Check Line syntax for Config File

Parameter: 	iType, int, input, define Variable Type
			bAlarm, bool, input, true -> PVSS alarm; false -> No PVSS alarm
			sDpe, string, input, string for the Dpe cell
			sName, string, input, string for the Name
			sState, string, input, string for the State
			sFC, string, input, string for the Fault Code 
			sDescription, string, input, string for the Description
			sEr, string, output, Pattern for the error 
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCMWServer_CheckLine(string sDevice, string sAlias, string sDpe, string sProperty, string sDirection, 
string sValue, string sTime, string &sEr)
{
int iEr, iFC, iEr_2;
string sFC_Temp;
dyn_string dsSplitHierarchy;
bool bValid;

	sEr="";
	if ((sDpe!="") && (sAlias=="")){
		//Case 1 : dpe="something" alias =
				if (!dpExists(sDpe)) sEr="Dpe";	
			}
		else 
			if ((sDpe!="") && (sAlias!="")){
		//Case 2 : dpe="ProcessInput.Post" alias ="Alias"
				sDpe = unGenericDpFunctions_dpAliasToName(sAlias)+sDpe;
				if (!dpExists(sDpe)) sEr="Dpe";	
			}
		
	//. Device	
	iEr=0;
	iEr=strpos(sDevice,"\"");
	iEr=strpos(sDevice,"'") + iEr;
	iEr=strpos(sDevice,">") + iEr;
	iEr=strpos(sDevice,"<") + iEr;
	iEr=strpos(sDevice,"~") + iEr; 
	if (iEr> -5 || sDevice =="ERROR")		
		{
		sEr="Device";
		return;
		}
	
	//. Property
	iEr=0;
	iEr=strpos(sProperty,"'") + iEr;
	iEr=strpos(sProperty,">") + iEr;
	iEr=strpos(sProperty,"<") + iEr;
	iEr=strpos(sProperty,"~") + iEr; 
	if (iEr> -4 || sProperty =="ERROR")	
	if (sProperty=="ERROR")		
		{
		sEr="Property";
		return;
		}
	
	//. Direction
	if ((sDirection!="in") && (sDirection!="out") && (sDirection!="in/out") )
		bValid=false;
	else bValid=true;	
	if ((!bValid) || (sDirection=="ERROR"))		
		{
		sEr="Direction";
		return;
		}
	
	//. Value
	if ((sValue!="boolean") && (sValue!="byte") && (sValue!="short") && (sValue!="long") && (sValue!="longlong")
	&& (sValue!="float") && (sValue!="double")&& (sValue!="string"))
		bValid=false;
	else bValid=true;	
	
	if ((!bValid) || (sValue=="ERROR"))		
	{
		sEr="Value";
		return;
	}
	
	//. Time
	if ((sTime!="longlongnano") && (sTime!="longlong")	&& (sTime!="double"))		bValid=false;
	else bValid=true;	
	
	if ((!bValid) || (sTime=="ERROR"))		
		{
		sEr="Time";
		return;
		}
}

// unCMWServer_getFromFile
/**
Purpose: Check Line syntax for Config File

Parameter: 	sConfigPath, string, input, Path
			Selected, dyn_string, output, array for the Selected column 
			Archive, dyn_string, output, array for the Archive column  
			Alias, dyn_string, output, array for the Alias column 
			Dpe, dyn_string, output, array for the Dpe column 
			Name, dyn_string, output, array for the Name column
			Description, dyn_string, output, array for the Description column
			Range, dyn_string, output, array for the Range column
			State, dyn_string, output, array for the State column
			FF, dyn_string, output, array for the FF column
			FM, dyn_string, output, array for the FM column
			FC, dyn_string, output, array for the FC column
			
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unCMWServer_getFromFile(string sConfigPath, dyn_string &Select, dyn_string &Device,  dyn_string &Alias, dyn_string &Dpe, 
							dyn_string &Property, dyn_string &Direction, dyn_string &Value, dyn_string &Time, dyn_int &Line, bool &bEr)
{

string fileInString, sSelect, sDevice, sAlias, sDpe, sProperty, sDirection, sValue, sTime, sPath;
int i, len, iLen_Range,imaxRange,j,len2,iNbLine,iStart ;
float fMax, fIndex;
dyn_string dsFileLines, dsSplitRange;
dyn_dyn_string ddsFile;
bool bChange, bDefined;

	iNbLine=0;
	if (isFunctionDefined("ctrlConfRead"))
	{
		ctrlConfRead(sConfigPath,ddsFile);
		len=dynlen(ddsFile);
		fMax=len;
		
		for (i=1;i<=len;i++) //read each statement
		{				
			int l=1;
			dsFileLines=ddsFile[i];
			len2=dynlen(dsFileLines);
			fIndex=i;
			progressBar.progress(fIndex/fMax*100);
			if(patternMatch("*[*]*", dsFileLines[l]))
			{
				bDefined=true;
//				sDevice=dsFileLines[l];
				sDevice=substr(dsFileLines[l],1,strlen(dsFileLines[l])-2);
				iNbLine++;
			} //End pattern type
			else //dpe
			{
				if(bDefined)
				{
					l++;
					if(patternMatch("*dpe=*", dsFileLines[l]))
					{
						sDpe=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
						l++;
						iNbLine++;
						if(patternMatch("*alias=*", dsFileLines[l]))
						{
							sAlias=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
							l++;			
							iNbLine++;
							if(patternMatch("*property=*", dsFileLines[l]))
							{
								sProperty=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
								l++;						
								iNbLine++;
								sSelect="X";
								if(patternMatch("*direction=*", dsFileLines[l]))
								{
									sDirection=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
									l++;	
									iNbLine++;	
								} //End pattern direction	
								else
								{
									sSelect="F";
									bEr=true;	
									sDirection="ERROR";
								}	
								if(patternMatch("*value=*", dsFileLines[l]))
								{
									sValue=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
									l++;	
									iNbLine++;	
								}	//End pattern value
								else
								{
									sSelect="F";
									bEr=true;	
									sValue="ERROR";
								}	
								if(patternMatch("*time=*", dsFileLines[l]))
								{
									sTime=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
									l++;	
									iNbLine++;	
								}//End pattern time
								else
								{
								 	sSelect="F";
								 	bEr=true;	
								 	sTime="ERROR";
								}
							} //End pattern property
							else
							{
								sSelect="F";
								sProperty="ERROR";
								sDirection="ERROR";
								sValue="ERROR";
								sTime="ERROR";
								bEr=true;
							}			
						} //End pattern alias
						else
						{
							sSelect="F";
							sAlias="ERROR";
							sProperty="ERROR";
							sDirection="ERROR";
							sValue="ERROR";
							sTime="ERROR";
							bEr=true;
						}
				
					}// End pattern dpe
					else //dpe ERROR
					{
						sSelect="F";
						sDpe="ERROR";
						sAlias="ERROR";
						sProperty="ERROR";
						sDirection="ERROR";
						sValue="ERROR";
						sTime="ERROR";
						bEr=true;	
					}
						dynAppend(Select,sSelect);
						dynAppend(Device,sDevice);
						dynAppend(Alias,sAlias);
						dynAppend(Dpe,sDpe);
						dynAppend(Property,sProperty);
						dynAppend(Direction,sDirection);
						dynAppend(Value,sValue);
						dynAppend(Time,sTime);
						dynAppend(Line,iNbLine);	
				}
				else //first element error
				{
					dynAppend(Select,"F");
					dynAppend(Device,"ERROR");
					dynAppend(Alias,"ERROR");
					dynAppend(Dpe,"ERROR");
					dynAppend(Line,iNbLine);		
					dynAppend(Property,"ERROR");
					dynAppend(Direction,"ERROR");
					dynAppend(Value,"ERROR");
					dynAppend(Time,"ERROR");
					bEr=true;
				}
			} //End else pattern type
		} //End for
	}//End if function defined
	else
	{
	DebugN("DLL not defined");
	bEr=true;
	}//End function not defined

}


// unCMWServer_Update_PVSS_Status
/**
Purpose: Start 
Parameter: 	sManager, string, input, Manager Name
			diState_PVSS, dyn_string, States of Manager in PVSS console
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCMWServer_Update_PVSS_Status(string sManager, int iState_PVSS)
{
int iPosition, iStatus, iRes;
dyn_string exceptionInfoTemp;

	unCMWServer_getPosition(sManager, iPosition);

	if (iState_PVSS!=g_diCMWServer_PVSS[iPosition])			//PVSS Status has changed
		{
		g_diCMWServer_PVSS[iPosition]=iState_PVSS;
		
		if (myManType()==UI_MAN) //IS-1745
			{//2.2 Get State
			unCMWServer_getStatus(iPosition, iStatus);
			
			//2.3 Update Table in Panel
			unCMWServer_setStatusTable(sManager, iPosition, iStatus, "");	
			}
		else
			{
			iRes=dpSet(getSystemName() + sManager + ".State.ev_MessageOut", g_diCMWServer_Message[iPosition]);
			}		
		}
}

// unCMWServer_Integrity
/**
Purpose: Start 
Parameter: 	sManager, string, input, Manager Name
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCMWServer_Integrity(string sManager, string sSystem)
{
int iPosition;

	unCMWServer_getPosition(sManager, iPosition);
	if (g_diCMWServer_Period[iPosition]<=0)
		g_diCMWServer_Period[iPosition]=c_Min_Rate;
	
	while (true) 	
		{
	  	unCMWServer_Check_Integrity(sManager, sSystem);
	  	unCMWServer_getPosition(sManager, iPosition);
	  	if (iPosition<1)
			delay(c_Min_Rate);
	  	else 
			delay(g_diCMWServer_Period[iPosition]); 
		} 
}

// unLaser_Check_Integrity
/**
Purpose: Function checks period of answer from Manager

Parameter: 	sMan, string, input, Manager Name
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unCMWServer_Check_Integrity(string sMan, string sSystem)
{
int iLastRefresh, iNow, iPeriod, iRes, iPosition, iMan_Num, iManager;
time tNow, tCMWServer_LastRefresh;
//string sSystem, sDatapoint_1, sDatapoint_2;
string sDatapoint_1, sDatapoint_2;
dyn_string exceptionInfo;

	tNow=getCurrentTime();
	unCMWServer_getPosition(sMan, iPosition);

	sDatapoint_1 = sSystem + sMan + ".Settings.man_Num";
	iRes=dpGet(sDatapoint_1, iMan_Num);

	if (iRes>=0)
		{
		if(dynContains(g_diCMWServer_Api, iMan_Num))
			{
			sDatapoint_2 = sSystem + sMan + ".Settings.man_Num:_online.._stime";
			iRes=dpGet(sDatapoint_2, tCMWServer_LastRefresh);
			if (iRes>=0)
				{
				//Get Time in seconds from 1970
				if (isFunctionDefined("ctrlTime"))
					{
					iNow=ctrlTime(tNow);		
					iLastRefresh=ctrlTime(tCMWServer_LastRefresh);
					}
				else
					{
					iNow=0;		
					iLastRefresh=0;
					fwException_raise(exceptionInfo, "ERROR", "unCMWServer_Save_Delay_Archive(): " + getCatStr(g_sCat_CMWServer,"DLLERROR"),"");
					}

				if (iNow!=0 && iLastRefresh!=0)
					iPeriod=iNow-iLastRefresh;		
				else
					iPeriod=g_diCMWServer_Period[iPosition] + 1;	//Error
				
				//DebugN(iPeriod + " >? " + g_diLaser_Period[iPosition]);
				if (iPeriod > g_diCMWServer_Period[iPosition])
					iManager=2;		//blocked
				else
					iManager=1;		//Started
				}
			else
				iManager=-1;		//?
			}
		else
			iManager=0;				//Stopped
		}
	else
		iManager=-1;				//?
			
	unCMWServer_Update_PVSS_Status(sMan, iManager);	
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity__CMWServer_CB
/**
Purpose:
Get Global Information

	@param sDp_1: string, input, data point name
	@param iPeriod: int, Period in minutes where CMWServer updates man_Num

Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/


unSystemIntegrity__CMWServer_CB(string sDp_1, int iPeriodMilli)
{
string sManager;
int iPosition;
int iPeriod = iPeriodMilli/1000;

	sManager=substr(sDp_1, strpos(sDp_1, c_CMWServer_Pattern), strlen(c_CMWServer_Pattern) + 2);
	
	unCMWServer_getPosition(sManager, iPosition);
	if (iPeriod<(c_Min_Rate/2))
		g_diCMWServer_Period[iPosition]=c_Min_Rate;
	else
		g_diCMWServer_Period[iPosition]= iPeriod * 2;
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity__CMWServer_CB_Api
/**
Purpose:
Get Api Connections

	@param sDp_1: string, input, data point name
	@param diApi: dyn_int, Api Connections
Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unSystemIntegrity__CMWServer_CB_Api(string sDp_1, dyn_int diApi)
{
	g_diCMWServer_Api=diApi;
}

//------------------------------------------------------------------------------------------------------------------------
unCMWServer_classByDevice(dyn_string &Select, dyn_string &Device,  dyn_string &Alias, dyn_string &Dpe, 
							dyn_string &Property, dyn_string &Direction, dyn_string &Value, dyn_string &Time, dyn_string &exceptionInfo_Temp)
{
dyn_string dsUniqueDevice;

dsUniqueDevice = Device;
dynUnique(dsUniqueDevice);
}

							
//@}



